/*
 *  Gtk-Parted - a front-end for libparted
 *  Copyright (C) 2000, 2001  Damien Genet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * main functions
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <getopt.h>

#include <parted/parted.h> 
#include <gtk/gtk.h>

#include "../misc/misc.h"

#include "gtk-parted.h"
#include "gtk-parted_infos.h"


static struct option options[] = {
	{"copyright",	0, NULL, 'c'},
	{"fstab-path",	1, NULL, 'f'},
	{"help",	0, NULL, 'h'},
	{"version",	0, NULL, 'V'},
	{NULL,		0, NULL, 0}
}; 


static GtkWidget*
create_dialogConfirmExit               (GtkWidget        *window,
                                        GdkEvent         *event,
                                        gpedInfo         *info)
{
	static GtkWidget	*dialogConfirmExit;

	g_return_val_if_fail (window, NULL);
	g_return_val_if_fail (GTK_IS_WINDOW (window), NULL);
	g_return_val_if_fail (info, NULL);
	
	if (dialogConfirmExit) {
		gdk_window_show (dialogConfirmExit->window);
		gdk_window_raise (dialogConfirmExit->window);
	} else if ( gped_get_changes (info) ) {
		dialogConfirmExit = ui_dialogConfirm_create (_("Changes not saved\nDo you really want to quit?"), GTK_SIGNAL_FUNC (gtk_main_quit));

		/*  Set dialogConfirmExit at NULL when destoyed */
		gtk_signal_connect (GTK_OBJECT (dialogConfirmExit), "destroy",
		                    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
		                    &dialogConfirmExit);
	} else
		gtk_main_quit ();
	
	return dialogConfirmExit;
}


/*
 * insert ctreeParted in a main window
 */
static GtkWidget*
create_main_window                     (gpedInfo         *info)
{
	GtkWidget	*window;
	
	g_return_val_if_fail (info, NULL);
	
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window), program_name);
	gtk_window_set_default_size (GTK_WINDOW (window), 500, 240);

	if ( !gped_insert (info, GTK_CONTAINER (window)) )
		return NULL;	

	gtk_signal_connect (GTK_OBJECT (window), "delete-event",
	                    GTK_SIGNAL_FUNC (create_dialogConfirmExit),
	                    info);
		
	gtk_widget_show_all (window);
	
	return window;
}

static void
_version                               (void)
{
	printf ("%s\n", program_name);
}

static void
_copyright                             (void)
{
	printf ("%s", _(copyright_msg));
}

static void
_usage                                 ()
{
	printf (_("Gtk-Parted - a front-end for libparted\n"
	          "Usage: gtk-parted [OPTION]... [DEVICE]...\n"
	          "\n"
	          "OPTIONs:\n"
	          " -c, --copyright\t\t"	"display the copyright and exit\n"
	          " -f, --fstab-path=PATH\t\t"	"use PATH as fstab\n"
	          " -h, --help\t\t\t"		"display this help and exit\n"
	          " -V, --version\t\t\t"	"display the version and exit\n"));
}

static gchar*
_parse_options                         (gint            *argc,
                                        gchar          **argv[])
{
	gint	 opt;
	gchar	*fstab_path = NULL;
	
	while ( (opt = getopt_long (*argc, *argv, "cf:hV", options, NULL)) != EOF ) {
		switch (opt) {
			case 'c':
				_copyright ();
				exit (0);
			case 'f':
				if (fstab_path)
					g_free (fstab_path);
				fstab_path = g_strdup (optarg);
				break;
			case 'h':
				_usage ();
				exit (0);
			case 'V':
				_version ();
				exit (0);
			default:
				g_printerr (_("Try %s -h for usage\n"),
				            *argv[0]);
				exit (-1);
		}
	}
	
	*argc -= optind;
	*argv += optind;

	return fstab_path;
}

static gpedInfo*
_init                                  (gint              argc,
                                        gchar            *argv[])
{
	gchar	*fstab_path;
	
	fstab_path = _parse_options (&argc, &argv);

	return gped_init (fstab_path, argc, argv);
}

int
main                                   (int               argc,
                                        char             *argv[])
{
	gpedInfo	*info;

#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
#endif

	/* init gtk */
	gtk_set_locale ();
	gtk_init (&argc, &argv);

	if ( !(info = _init (argc, argv)) )
		exit (1);
	gtk_thread_start ();
	
	/* main loop */
	if ( !create_main_window (info) )
		exit (1);
	gtk_main ();
	
	gtk_thread_stop ();
	gped_done ();
		
	return 0;
}

