/*
 *  Gtk-Parted - a front-end for libparted
 *  Copyright (C) 2000, 2001  Damien Genet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * popup menu functions
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include <parted/parted.h>
#include <gtk/gtk.h>

#include "../misc/i18n.h"

#include "gped.h"
#include "ctreeParted_callbacks.h"
#include "popup.h"
#include "popup_callbacks.h"


/*
 * disable menu entries that are not possible with the part selected
 */

typedef struct _gpedMenuEntriesInfo gpedMenuInfo;
struct _gpedMenuEntriesInfo
{
	GtkWidget	*properties;
	GtkWidget	*add;
	GtkWidget	*add_primary;
	GtkWidget	*add_logical;
	GtkWidget	*add_extended;
	GtkWidget	*delette;
	GtkWidget	*advanced;
	GtkWidget	*maximise;
	GtkWidget	*minimise_ext;
	GtkWidget	*check;
	GtkWidget	*create;
	GtkWidget	*del_all;
	GtkWidget	*write;
};

static void
_nothing_selected_disable_entries      (gpedMenuInfo    *menu,
                                        gpedInfo        *info)
{	
	g_return_if_fail (menu);
	g_return_if_fail (info);
	
	gtk_widget_set_sensitive (GTK_WIDGET (menu->properties), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->add), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->delette), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->advanced), FALSE);
}



static void
_device_selected_disable_entries       (gpedMenuInfo    *menu,
                                        gpedInfo        *info,
                                        GtkCTreeNode    *node)
{	
	g_return_if_fail (menu);
	g_return_if_fail (info);

	gtk_widget_set_sensitive (GTK_WIDGET (menu->properties), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->add), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->delette), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->advanced), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->maximise), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->check), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->create), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->minimise_ext), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->del_all), FALSE);
}

static void
_disk_selected_disable_entries         (gpedMenuInfo    *menu,
                                        gpedInfo        *info,
                                        GtkCTreeNode    *node)
{
	PedDisk		*disk;
	
	g_return_if_fail (menu);
	g_return_if_fail (info);

	gtk_widget_set_sensitive (GTK_WIDGET (menu->properties), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->add), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->delette), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->advanced), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->maximise), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->check), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->create), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->del_all), TRUE);

	if ( !(disk = ((gpedDeviceInfo*) (gtk_ctree_node_get_row_data (GTK_CTREE (info->ctreeParted), GTK_CTREE_NODE (node))))->disk) )
		g_assert_not_reached ();	/* should never happen */
	if (ped_disk_extended_partition (disk))
		gtk_widget_set_sensitive (GTK_WIDGET (menu->minimise_ext), TRUE);
	else
		gtk_widget_set_sensitive (GTK_WIDGET (menu->minimise_ext), FALSE);
}

static void
_ext_selected_disable_entries          (gpedMenuInfo    *menu,
                                        gpedInfo        *info)
{	
	g_return_if_fail (menu);
	g_return_if_fail (info);
	
	gtk_widget_set_sensitive (GTK_WIDGET (menu->properties), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->add), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->delette), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->advanced), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->maximise), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->minimise_ext), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->check), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->create), FALSE);
}

static guint
primary_partition_count                (PedDisk         *disk)
{
	PedPartition	*part = NULL;
	guint		 nb_primary = 0;
	
	g_return_val_if_fail (disk, 0);
	
	while ( (part = ped_disk_next_partition (disk, part)) ) {
		if (part->type == PED_PARTITION_PRIMARY)
			nb_primary++;
	}
	
	return nb_primary;
}

static void
_free_space_selected_disable_entries   (gpedMenuInfo    *menu,
                                        gpedInfo        *info,
                                        PedPartition    *part)
{	
	g_return_if_fail (menu);
	g_return_if_fail (info);

	gtk_widget_set_sensitive (GTK_WIDGET (menu->properties), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->add), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->delette), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->maximise), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->check), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->create), FALSE);

	if (primary_partition_count (part->geom.disk) < ped_disk_get_max_primary_partition_count (part->geom.disk)
	    && !(part->type & PED_PARTITION_LOGICAL) )
		gtk_widget_set_sensitive (GTK_WIDGET (menu->add_primary), TRUE);
	else
		gtk_widget_set_sensitive (GTK_WIDGET (menu->add_primary), FALSE);

	if (part->type & PED_PARTITION_LOGICAL)
		gtk_widget_set_sensitive (GTK_WIDGET (menu->add_logical), TRUE);
	else
		gtk_widget_set_sensitive (GTK_WIDGET (menu->add_logical), FALSE);

	if (!ped_disk_extended_partition (part->geom.disk)
	    && primary_partition_count (part->geom.disk) < ped_disk_get_max_primary_partition_count (part->geom.disk)
	    && !(part->type & PED_PARTITION_LOGICAL) )
		gtk_widget_set_sensitive (GTK_WIDGET (menu->add_extended), TRUE);
	else
		gtk_widget_set_sensitive (GTK_WIDGET (menu->add_extended), FALSE);

}

static void
_used_part_selected_disable_entries    (gpedMenuInfo    *menu,
                                        gpedInfo        *info,
                                        PedPartition    *part)
{	
	g_return_if_fail (menu);
	g_return_if_fail (info);
	
	gtk_widget_set_sensitive (GTK_WIDGET (menu->properties), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->add), FALSE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->delette), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->maximise), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->create), TRUE);
	if (part->fs_type)
		gtk_widget_set_sensitive (GTK_WIDGET (menu->check), TRUE);
	else
		gtk_widget_set_sensitive (GTK_WIDGET (menu->check), FALSE);
}

static void
_part_selected_disable_entries         (gpedMenuInfo    *menu,
                                        gpedInfo        *info,
                                        GtkCTreeNode    *node)
{	
	PedPartition	*part;
	
	g_return_if_fail (menu);
	g_return_if_fail (info);
	g_return_if_fail (node);

	if ( !(part = gtk_ctree_node_get_row_data (GTK_CTREE (info->ctreeParted), GTK_CTREE_NODE (node)) ))
		g_assert_not_reached ();	/* should never happen */

	gtk_widget_set_sensitive (GTK_WIDGET (menu->advanced), TRUE);
	gtk_widget_set_sensitive (GTK_WIDGET (menu->del_all), TRUE);
	if (ped_disk_extended_partition (part->geom.disk))
		gtk_widget_set_sensitive (GTK_WIDGET (menu->minimise_ext), TRUE);
	else
		gtk_widget_set_sensitive (GTK_WIDGET (menu->minimise_ext), FALSE);
		
	if (part->type & PED_PARTITION_FREESPACE)
		_free_space_selected_disable_entries (menu, info, part);
	else
		_used_part_selected_disable_entries (menu, info, part);
}

static void
_disable_write_entry                   (gpedMenuInfo    *menu,
                                        gpedInfo        *info)
{	
	g_return_if_fail (menu);
	g_return_if_fail (info);

	if ( gped_get_changes (info) )
		gtk_widget_set_sensitive (GTK_WIDGET (menu->write), TRUE);
	else
		gtk_widget_set_sensitive (GTK_WIDGET (menu->write), FALSE);
}

static gpedMenuInfo*
_gpedMenuInfo_new                      (void)
{
	gpedMenuInfo	*menu;

	 if ( !(menu = g_malloc (sizeof (gpedMenuInfo))) ) {
		g_printerr ("%s: allocating gpedMenuInfo.\n",
		            g_strerror (errno));
		return NULL;
	}

	return menu;
}

gpedMenuInfo*
_menu_info_get                         (GtkItemFactory  *item_factory)
{
	static gpedMenuInfo	*menu = NULL;
	
	g_return_val_if_fail (item_factory, NULL);
	g_return_val_if_fail (GTK_IS_ITEM_FACTORY (item_factory), NULL);
	
	if (menu)
		return menu;
	
	if ( !(menu = _gpedMenuInfo_new ()) )
		return NULL;
	
	menu->properties   = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_PROPERTIES);
	menu->add          = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_ADD);
	menu->add_primary  = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_ADD_PRIMARY);
	menu->add_logical  = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_ADD_LOGICAL);
	menu->add_extended = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_ADD_EXTENDED);
	menu->delette      = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_DEL);
	menu->advanced     = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_ADVANCED);
	menu->maximise     = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_MAXIMISE_PART);
	menu->minimise_ext = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_MINIMISE_EXT);
	menu->check        = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_CHECK_FS);
	menu->create       = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_CREATE_FS);
	menu->del_all      = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_DEL_ALL);
	menu->write        = gtk_item_factory_get_item_by_action (GTK_ITEM_FACTORY (item_factory),
	                                                          POPUP_WRITE);
	
	return menu;
}

void
popup_disable_unavailable_entries      (GtkItemFactory  *item_factory,
                                        gpedInfo        *info)
{
	GtkCTreeNode		*node = NULL;
	gpedMenuInfo		*menu;
	
	g_return_if_fail (item_factory);
	g_return_if_fail (GTK_IS_ITEM_FACTORY (item_factory));
	g_return_if_fail (info);
	
	menu = _menu_info_get (GTK_ITEM_FACTORY (item_factory));
	
	/*
	 * find selected node
	 */
	if (GTK_CLIST (info->ctreeParted)->selection)	/* a node is selected */
		node = GTK_CTREE_NODE (GTK_CLIST (info->ctreeParted)->selection->data);
	
	if (!node)					/* nothing selected */
		_nothing_selected_disable_entries (menu, info);
	else if (!GTK_CTREE_ROW (node)->parent		/* device */
	           && !((gpedDeviceInfo*) (gtk_ctree_node_get_row_data (GTK_CTREE (info->ctreeParted), GTK_CTREE_NODE (node))))->disk )
		_device_selected_disable_entries (menu, info, node);
	else if (!GTK_CTREE_ROW (node)->parent)		/* disk */
		_disk_selected_disable_entries (menu, info, node);
	else if (GTK_CTREE_ROW (node)->children)	/* ext part */
		_ext_selected_disable_entries (menu, info);
	else						/* part */
		_part_selected_disable_entries (menu, info, node);
		
	_disable_write_entry (menu, info);
}


/*
 * GUI
 */

/*
 * create popup menu
 */
GtkItemFactory*
_create                                (gpedInfo         *info)
{
	GtkItemFactory	*item_factory;
	GtkAccelGroup	*accel_group;
	
	/*
	 * entries
	 */
	GtkItemFactoryEntry entries[] = {
		{_("/_Properties"),		NULL,	on_menu_popup_selected,
			POPUP_PROPERTIES,	"<Item>"},
		{_("/_Add Partition"),		NULL,	NULL,
			POPUP_ADD,		"<Branch>"},
		{_("/Add Partition/Primary"),	NULL, on_menu_popup_selected,
			POPUP_ADD_PRIMARY,	"<Item>"},
		{_("/Add Partition/Logical"),	NULL, on_menu_popup_selected,
			POPUP_ADD_LOGICAL,	"<Item>"},
		{_("/Add Partition/Extended"),	NULL, on_menu_popup_selected,
			POPUP_ADD_EXTENDED,	"<Item>"},
		{_("/_Delete Partition"),	NULL,	on_menu_popup_selected,
			POPUP_DEL,		"<Item>"},
		{_("/Ad_vanced"),		NULL,	NULL,
			POPUP_ADVANCED,		"<Branch>"},
		{_("/Advanced/Ma_ximise Partition Only"), NULL, on_menu_popup_selected,
			POPUP_MAXIMISE_PART,	"<Item>"},
		{_("/Advanced/Mi_nimise Extended Partition"), NULL, on_menu_popup_selected,
			POPUP_MINIMISE_EXT,	"<Item>"},
		{_("/Advanced/Create FileSystem"), NULL, NULL,
			POPUP_CREATE_FS,	"<Branch>"},
		{_("/Advanced/Create FileSystem/ext2"), NULL,	on_menu_popup_selected,
			POPUP_CREATE_FS_ext2,	"<Item>"},
		{_("/Advanced/Create FileSystem/FAT"), NULL,	on_menu_popup_selected,
			POPUP_CREATE_FS_FAT,	"<Item>"},
		{_("/Advanced/Create FileSystem/linux-swap"), NULL,	on_menu_popup_selected,
			POPUP_CREATE_FS_linux_swap,	"<Item>"},
		{_("/Advanced/_Check FileSystem"), NULL,	on_menu_popup_selected,
			POPUP_CHECK_FS,		"<Item>"},
		{_("/Advanced/Sep"), NULL, NULL, 0, "<Separator>"},
		{_("/Advanced/Delete All Partitions"),	NULL,	on_menu_popup_selected,
			POPUP_DEL_ALL,		"<Item>"},
		{_("/Advanced/Reset Disk Label"), NULL, NULL,
			POPUP_RESET,		"<Branch>"},
		{_("/Advanced/Reset Disk Label/MSDos"), NULL,	on_menu_popup_selected,
			POPUP_RESET_DOS,	"<Item>"},
		{_("/Advanced/Reset Disk Label/Loopback"), NULL,	on_menu_popup_selected,
			POPUP_RESET_LOOP,	"<Item>"},
		{_("/Advanced/Reset Disk Label/Mac"), NULL,	on_menu_popup_selected,
			POPUP_RESET_MAC,	"<Item>"},
		{_("/Advanced/Reset Disk Label/PC98"), NULL,	on_menu_popup_selected,
			POPUP_RESET_PC98,	"<Item>"},
		{ "/Sep", NULL, NULL, 0, "<Separator>"},
		{_("/Re_fresh"),		NULL,	on_menu_popup_selected,
			POPUP_REFRESH,		"<Item>"},
		{_("/Write changes"),		NULL,	on_menu_popup_selected,
			POPUP_WRITE,		"<Item>"},
		{ "/Sep", NULL, NULL, 0, "<Separator>"},
		{_("/A Propos"),		NULL,	on_menu_popup_selected,
			POPUP_APROPOS,		"<Item>"}
	};
	
	g_return_val_if_fail (info, NULL);
	
	accel_group = gtk_accel_group_new ();
	item_factory = gtk_item_factory_new (GTK_TYPE_MENU,
	                                     "<MAIN>",
	                                     accel_group);
	gtk_item_factory_create_items (GTK_ITEM_FACTORY (item_factory), 26,
	                               entries, info);
	gtk_accel_group_attach (accel_group, GTK_OBJECT (info->ctreeParted));

	return item_factory;
}

GtkItemFactory*
popup_insert                           (gpedInfo         *info)
{
	GtkItemFactory	*item_factory;

	g_return_val_if_fail (info, NULL);

	if ( !(item_factory = _create (info)) )
		return NULL;
	
	gtk_signal_connect (GTK_OBJECT (info->ctreeParted), "button_press_event",
	                    GTK_SIGNAL_FUNC (on_ctreeParted_button_press_event),
	                    info );
	
	return item_factory;
}
